@@ -16,20 +16,20 @@
 
 """Python client library for the Facebook Platform.
 
-This client library is designed to support the Graph API and the official
-Facebook JavaScript SDK, which is the canonical way to implement
-Facebook authentication. Read more about the Graph API at
+This client library is designed to support the Graph API and the
+official Facebook JavaScript SDK, which is the canonical way to
+implement Facebook authentication. Read more about the Graph API at
 http://developers.facebook.com/docs/api. You can download the Facebook
 JavaScript SDK at http://github.com/facebook/connect-js/.
 
 If your application is using Google AppEngine's webapp framework, your
 usage of this module might look like this:
 
-    user = facebook.get_user_from_cookie(self.request.cookies, key, secret)
-    if user:
-        graph = facebook.GraphAPI(user["access_token"])
-        profile = graph.get_object("me")
-        friends = graph.get_connections("me", "friends")
+user = facebook.get_user_from_cookie(self.request.cookies, key, secret)
+if user:
+    graph = facebook.GraphAPI(user["access_token"])
+    profile = graph.get_object("me")
+    friends = graph.get_connections("me", "friends")
 
 """
 
@@ -41,6 +41,7 @@ import hashlib
 import hmac
 import base64
 import logging
+import socket
 
 # Find a JSON parser
 try:
@@ -62,15 +63,15 @@ except ImportError:
 class GraphAPI(object):
     """A client for the Facebook Graph API.
 
-    See http://developers.facebook.com/docs/api for complete documentation
-    for the API.
+    See http://developers.facebook.com/docs/api for complete
+    documentation for the API.
 
-    The Graph API is made up of the objects in Facebook (e.g., people, pages,
-    events, photos) and the connections between them (e.g., friends,
-    photo tags, and event RSVPs). This client provides access to those
-    primitive types in a generic way. For example, given an OAuth access
-    token, this will fetch the profile of the active user and the list
-    of the user's friends:
+    The Graph API is made up of the objects in Facebook (e.g., people,
+    pages, events, photos) and the connections between them (e.g.,
+    friends, photo tags, and event RSVPs). This client provides access
+    to those primitive types in a generic way. For example, given an
+    OAuth access token, this will fetch the profile of the active user
+    and the list of the user's friends:
 
        graph = facebook.GraphAPI(access_token)
        user = graph.get_object("me")
@@ -80,15 +81,17 @@ class GraphAPI(object):
     by the API at http://developers.facebook.com/docs/reference/api/.
 
     You can obtain an access token via OAuth or by using the Facebook
-    JavaScript SDK. See http://developers.facebook.com/docs/authentication/
-    for details.
+    JavaScript SDK. See
+    http://developers.facebook.com/docs/authentication/ for details.
 
     If you are using the JavaScript SDK, you can use the
     get_user_from_cookie() method below to get the OAuth access token
     for the active user from the cookie saved by the SDK.
+
     """
-    def __init__(self, access_token=None):
+    def __init__(self, access_token=None, timeout=None):
         self.access_token = access_token
+        self.timeout = timeout
 
     def get_object(self, id, **args):
         """Fetchs the given object from the graph."""
@@ -97,8 +100,8 @@ class GraphAPI(object):
     def get_objects(self, ids, **args):
         """Fetchs all of the given object from the graph.
 
-        We return a map from ID to object. If any of the IDs are invalid,
-        we raise an exception.
+        We return a map from ID to object. If any of the IDs are
+        invalid, we raise an exception.
         """
         args["ids"] = ",".join(ids)
         return self.request("", args)
@@ -121,13 +124,15 @@ class GraphAPI(object):
             post = feed["data"][0]
             graph.put_object(post["id"], "comments", message="First!")
 
-        See http://developers.facebook.com/docs/api#publishing for all of
-        the supported writeable objects.
+        See http://developers.facebook.com/docs/api#publishing for all
+        of the supported writeable objects.
+
+        Certain write operations require extended permissions. For
+        example, publishing to a user's feed requires the
+        "publish_actions" permission. See
+        http://developers.facebook.com/docs/publishing/ for details
+        about publishing permissions.
 
-        Certain write operations require extended permissions. For example,
-        publishing to a user's feed requires the "publish_actions" permission. See
-        http://developers.facebook.com/docs/publishing/ for details about
-        publishing permissions.
         """
         assert self.access_token, "Write operations require an access token"
         return self.request(parent_object + "/" + connection_name,
@@ -139,8 +144,8 @@ class GraphAPI(object):
         We default to writing to the authenticated user's wall if no
         profile_id is specified.
 
-        attachment adds a structured attachment to the status message being
-        posted to the Wall. It should be a dictionary of the form:
+        attachment adds a structured attachment to the status message
+        being posted to the Wall. It should be a dictionary of the form:
 
             {"name": "Link name"
              "link": "http://www.example.com/",
@@ -165,11 +170,13 @@ class GraphAPI(object):
         self.request(id, post_args={"method": "delete"})
 
     def put_photo(self, image, message=None, album_id=None, **kwargs):
-        """Uploads an image using multipart/form-data
+        """Uploads an image using multipart/form-data.
+
         image=File like object for the image
         message=Caption for your image
         album_id=None posts to /me/photos which uses or creates and uses
         an album for your application.
+
         """
         object_id = album_id or "me"
         #it would have been nice to reuse self.request;
@@ -206,10 +213,14 @@ class GraphAPI(object):
 
     # based on: http://code.activestate.com/recipes/146306/
     def _encode_multipart_form(self, fields):
-        """Fields are a dict of form name-> value
-        For files, value should be a file object.
-        Other file-like objects might work and a fake name will be chosen.
-        Return (content_type, body) ready for httplib.HTTP instance
+        """Encode files as 'multipart/form-data'.
+
+        Fields are a dict of form name-> value. For files, value should
+        be a file object. Other file-like objects might work and a fake
+        name will be chosen.
+
+        Returns (content_type, body) ready for httplib.HTTP instance.
+
         """
         BOUNDARY = '----------ThIs_Is_tHe_bouNdaRY_$'
         CRLF = '\r\n'
@@ -243,8 +254,10 @@ class GraphAPI(object):
     def request(self, path, args=None, post_args=None):
         """Fetches the given path in the Graph API.
 
-        We translate args to a valid query string. If post_args is given,
-        we send a POST request to the given path with the given arguments.
+        We translate args to a valid query string. If post_args is
+        given, we send a POST request to the given path with the given
+        arguments.
+
         """
         args = args or {}
 
@@ -256,11 +269,16 @@ class GraphAPI(object):
         post_data = None if post_args is None else urllib.urlencode(post_args)
         try:
             file = urllib2.urlopen("https://graph.facebook.com/" + path + "?" +
-                                  urllib.urlencode(args), post_data)
+                    urllib.urlencode(args), post_data, timeout=self.timeout)
         except urllib2.HTTPError, e:
             response = _parse_json(e.read())
             raise GraphAPIError(response)
-
+        except TypeError:
+            # Timeout support for Python <2.6
+            if self.timeout:
+                socket.setdefaulttimeout(self.timeout)
+            file = urllib2.urlopen("https://graph.facebook.com/" + path + "?" +
+                                    urllib.urlencode(args), post_data)
         try:
             fileInfo = file.info()
             if fileInfo.maintype == 'text':
@@ -284,8 +302,10 @@ class GraphAPI(object):
     def api_request(self, path, args=None, post_args=None):
         """Fetches the given path in the Graph API.
 
-        We translate args to a valid query string. If post_args is given,
-        we send a POST request to the given path with the given arguments.
+        We translate args to a valid query string. If post_args is
+        given, we send a POST request to the given path with the given
+        arguments.
+
         """
         args = args or {}
         if self.access_token:
@@ -303,8 +323,17 @@ class GraphAPI(object):
         else:
             args["format"] = "json-strings"
         post_data = None if post_args is None else urllib.urlencode(post_args)
-        file = urllib.urlopen("https://api.facebook.com/method/" + path + "?" +
-                              urllib.urlencode(args), post_data)
+        try:
+            file = urllib.urlopen("https://api.facebook.com/method/" + path +
+                    "?" + urllib.urlencode(args),
+                    post_data, timeout=self.timeout)
+        except TypeError:
+            # Timeout support for Python <2.6
+            if self.timeout:
+                socket.setdefaulttimeout(self.timeout)
+            file = urllib.urlopen("https://api.facebook.com/method/" + path +
+                    "?" + urllib.urlencode(args), post_data)
+
         try:
             response = _parse_json(file.read())
         finally:
@@ -315,10 +344,7 @@ class GraphAPI(object):
 
     def fql(self, query, args=None, post_args=None):
         """FQL query.
-        Two reasons to have this method:
-        1. Graph api does not expose some info fields of a user, e.g.
-            a user's networks/affiliations, we have to fall back to old api.
-        2. FQL is a strong tool.
+
         Example query: "SELECT affiliations FROM user WHERE uid = me()"
 
         """
@@ -343,9 +369,18 @@ class GraphAPI(object):
 
         args["format"] = "json"
 
-        file = urllib2.urlopen("https://api.facebook.com/method/" +
+        try:
+            file = urllib2.urlopen("https://api.facebook.com/method/" +
                                fql_method + "?" + urllib.urlencode(args),
-                               post_data)
+                               post_data, timeout=self.timeout)
+        except TypeError:
+            # Timeout support for Python <2.6
+            if self.timeout:
+                socket.setdefaulttimeout(self.timeout)
+            file = urllib2.urlopen("https://api.facebook.com/method/" +
+                           fql_method + "?" + urllib.urlencode(args),
+                           post_data)
+
         try:
             content = file.read()
             response = _parse_json(content)
@@ -426,7 +461,9 @@ def get_user_from_cookie(cookies, app_id, app_secret):
 
     Download the official Facebook JavaScript SDK at
     http://github.com/facebook/connect-js/. Read more about Facebook
-    authentication at http://developers.facebook.com/docs/authentication/.
+    authentication at
+    http://developers.facebook.com/docs/authentication/.
+
     """
     cookie = cookies.get("fbsr_" + app_id, "")
     if not cookie:
@@ -449,6 +486,7 @@ def parse_signed_request(signed_request, app_secret):
     your application, as well as any information requested.
 
     If the signed_request is malformed or corrupted, False is returned.
+
     """
     try:
         l = signed_request.split('.', 2)
@@ -469,6 +507,11 @@ def parse_signed_request(signed_request, app_secret):
     if data.get('algorithm', '').upper() != 'HMAC-SHA256':
         return False
 
+    # HMAC can only handle ascii (byte) strings
+    # http://bugs.python.org/issue5285
+    app_secret = app_secret.encode('ascii')
+    payload = payload.encode('ascii')
+
     expected_sig = hmac.new(app_secret,
                             msg=payload,
                             digestmod=hashlib.sha256).digest()
@@ -478,19 +521,21 @@ def parse_signed_request(signed_request, app_secret):
     return data
 
 
-def auth_url(app_id, canvas_url, perms=None):
+def auth_url(app_id, canvas_url, perms=None, state=None):
     url = "https://www.facebook.com/dialog/oauth?"
     kvps = {'client_id': app_id, 'redirect_uri': canvas_url}
     if perms:
         kvps['scope'] = ",".join(perms)
+    if state:
+        kvps['state'] = state
     return url + urllib.urlencode(kvps)
 
 
 def get_access_token_from_code(code, redirect_uri, app_id, app_secret):
-    """
-    Get a user-specific access token from the "code" returned from a Facebook
-    OAuth dialog. Returns a dict containing the access token and its expiration
-    date (if applicable).
+    """Get an access token from the "code" returned from an OAuth dialog.
+
+    Returns a dict containing the user-specific access token and its
+    expiration date (if applicable).
 
     """
     args = {
@@ -499,8 +544,8 @@ def get_access_token_from_code(code, redirect_uri, app_id, app_secret):
         "client_id": app_id,
         "client_secret": app_secret,
     }
-    # We would use GraphAPI.request() here, except for that the fact that the
-    # response is a key-value pair, and not JSON.
+    # We would use GraphAPI.request() here, except for that the fact
+    # that the response is a key-value pair, and not JSON.
     response = urllib.urlopen("https://graph.facebook.com/oauth/access_token" +
                               "?" + urllib.urlencode(args)).read()
     query_str = parse_qs(response)
@@ -515,13 +560,14 @@ def get_access_token_from_code(code, redirect_uri, app_id, app_secret):
 
 
 def get_app_access_token(app_id, app_secret):
-    """
-    Get the access_token for the app that can be used for insights and
-    creating test users.
+    """Get the access_token for the app.
+
+    This token can be used for insights and creating test users.
 
     app_id = retrieved from the developer page
     app_secret = retrieved from the developer page
-    returns the application access_token
+
+    Returns the application access_token.
 
     """
     # Get an app access token
